# ______________________________________________________________________________
####                 Script METODOS DE INTERPOLACION                        ####
#
#      1) Preparacion de Datos: mapa de coberturas, archivo de datos,
#         objetos SpatialpointDataFrame,
#         Grid en donde se realizara la interpolacion y otros
#      2) Analisis de tendencias de superficies (polinomios de 1er y 2do orden)
#      3) Prediccion global usando modelos de clasificación
#      4) Poligonos de Thiessen
#      5) Distancia inversa ponderada (IDW)
#      6) interpolacion con Kriging
#      7) Interpolacion dentro de estratos
#         A.-Distancia inversa (IDW) dentro de estratos
#         B.- Interpolacion con kriging dentro de estratos
#      9) Regresion con kriging
#      10) Random forest de regresion con Kriging
#                                                                    Marzo-2023
#_______________________________________________________________________________

#_______________________________________________________________________________
# 1.- Preparación de datos                                                  ####
#_______________________________________________________________________________

##### A.- Leer datos del mapa de coberturas y obtener mascaras               ####
#_______________________________________________________________________________

# Definir directorio de trabajo, este debe contener los archivos de datos y el mapa 
# con el area de estudio delimitada
setwd("D:/met_int")
getwd()
folder <- getwd()

# Abrir los siguientes paquetes
library(sp)
library(raster)
library(ggplot2)
library(tmap)
library(tiff)

#Leer mapa de coberturas y reclasificarlo para crear una mascara
clases <- raster("sup_class_filter.tif")

# Asignar las coordenadas del del mapa de cobertura con UTM n16 WGS84
clases.proj <- "+proj=utm +zone=16 +datum=WGS84 +units=m +no_defs"
crs(clases) <- CRS(clases.proj)
clases

#opcion sencilla para graficar el mapa
#my_col <- c("black", "darkgreen","green", "greenyellow", "forestgreen", "orange", "yellow4")
#plot (clases,legend = TRUE, col = my_col)

#reclasificar el mapa de categorias
m0 <- c(0,0,1,1,2,2,3,3,4,4,5,5,6,6)
mcmat <- matrix(m0, ncol = 2, byrow = TRUE)

rveg <- reclassify(clases, mcmat)

#mapa de con los tipos de vegetación
#tm_raster(clases) +
tm1 <- tm_shape(rveg) +
  tm_raster(style = "cat",
            labels = c("Otras coberturas","Selva madura","Selva inter 2",
                       "Selva inter 1","Selva joven", "Bosque bajo", "Sabana"),
            palette = c("black", "darkgreen", "forestgreen","green",
                        "greenyellow","orange", "yellow4"),
            title = "Tipos de vegetación") +
  tm_grid() +
  tm_layout(legend.outside = TRUE,legend.position = c("right", "bottom"))
tm1

#crear el directorio de salida para guardar los mapas
dir.MAPS <-  file.path(folder,"Maps") #Definir la carpeta a usar
#Evaluar si ya existe la carpeta nombrada
ifelse(dir.exists(dir.MAPS), dir.MAPS,
          #Si no existe se crea la carpeta
          dir.create(dir.MAPS))


#imprimir mapa
tmap_save(tm1, filename = paste(dir.MAPS,"map_cober.tiff", sep = "/"), 
          width = 1720, height = 1070, asp = 0)

#Creacion de una mascara bosque no bosque
m <- c(0, 1, 0,1, 7, 1)
rclmat <- matrix(m, ncol = 3, byrow = TRUE)
rc <- reclassify(rveg, rclmat, right = FALSE )

#Mapa de la mascara
tm1 <- tm_shape(rc) +
  tm_raster(style = "cat",
            labels = c("No Bosque", "Bosque"),
            palette = c("white", "darkgreen"),
            title = "Tipos de vegetación") +
  tm_grid() +
  tm_layout(legend.outside = TRUE,legend.position = c("right", "bottom"))
tm1

#imprimir mapa
tmap_save(tm1, filename = paste(dir.MAPS,"map_bosque.tiff", sep = "/"),
          width = 1720, height = 1070, asp = 0)

#reclasificar las caterorias de vegetación la clase selva 
ms <- c(0,1,0,1,5,1,5,7,0)
rclmats <- matrix(ms, ncol = 3, byrow = TRUE)
rs <- reclassify(rveg, rclmats, right = FALSE)

#Mapa de la mascara de selva
tm1 <- tm_shape(rs) +
  tm_raster(style = "cat",
            labels = c("Otros", "Selva"),
            palette = c("white", "darkgreen"),
            title = "Tipos de vegetación") +
  tm_grid() +
  tm_layout(legend.outside = TRUE,legend.position = c("right", "bottom"))
tm1

#imprimir mapa
tmap_save(tm1, filename = paste(dir.MAPS,"map_selvas.tiff", sep = "/"), 
          width = 1720, height = 1070, asp = 0)

#reclasificar las caterorias de vegetación la clase selva baja inundable
mb <- c(0,5,0,5,7,1)
rclmatb <- matrix(mb, ncol = 3, byrow = TRUE)

rb <- reclassify(rveg, rclmatb, right = FALSE)

#Mapa de la mascara de selva baja inundable
tm1 <- tm_shape(rb) +
  tm_raster(style = "cat",
            labels = c("Otros", "Bosque bajo"),
            palette = c("white", "darkgreen"),
            title = "Tipos de vegetación") +
  tm_grid() +
  tm_layout(legend.outside = TRUE,legend.position = c("right", "bottom"))
tm1

#imprimir mapa
tmap_save(tm1, filename = paste(dir.MAPS, "map_bajos.tiff", sep = "/"), 
          width = 1720, height = 1070, asp = 0)


#####  B.- Leer archivo de datos y Crear objetos SpatialpointDataFrame       ####
#_______________________________________________________________________________
#
library(spatstat)

# Cargar el archivo que contenga los datos de campo de la variable de interes con coordenadas
datos <- read.csv("dat.csv")
head(datos)

#En el paquete spatstat todos los objetos se asumne que son planos. 
#Esto significa que spatstat no esta diseñado para trabajar directamente con 
#coordenadas geograficas (longituded y latituded). 

# Eliminar del dataframe datos las variables que no correspondan a las coordenadas
coor <- datos[ , c(2,3)]
#convertir puntos de un dataframe a SpatialPoints 
sp.points <-  SpatialPoints(coor)
proj4string(sp.points) <- "+proj=utm +zone=16 +units=m +ellps=WGS84 +datum=WGS84 +no_defs"

#convertir Spatial points to SpatialpointDataFrame
spdf = SpatialPointsDataFrame(sp.points, datos)

#mapa de las parcelas de todos los tipos de vegetación
my_pal <- c("yellow4", "greenyellow", "green","forestgreen","darkgreen")
tm1 <- tm_shape(spdf) +
  tm_dots(col = "Num_esp", palette = my_pal,
  title = "Número de especies", size = 0.3) +
  tm_text("Num_esp", just = "left", xmod = .5, size = 0.7) +
  tm_grid() +
  tm_layout(legend.outside = TRUE, legend.position = c("right", "bottom"))
tm1

#imprimir mapa
tmap_save(tm1, filename = paste(dir.MAPS,"map_parcelas.tiff", sep = "/"), 
          width = 1720, height = 1070, asp = 0)

#crear subconjuntos de datos para los estratos selva y selvabaja inundable
spdf_sel = spdf[spdf$t_veg <= 4,]
spdf_baj = spdf[spdf$t_veg >= 5,]

#mapa de las parcelas de la selva
my_pal <- c("greenyellow", "green","forestgreen","darkgreen")
tm1 <- tm_shape(spdf_sel) +
  tm_dots(col = "Num_esp", palette = my_pal,
          title = "Número de especies", size = 0.3) +
  tm_text("Num_esp", just = "left", xmod = .5, size = 0.7) +
  tm_grid() +
  tm_layout(legend.outside = TRUE, legend.position = c("right", "bottom"))
tm1

#imprimir mapa
tmap_save(tm1, filename = paste(dir.MAPS, "map_parc_sevas.tiff", sep = "/"),
          width = 1720, height = 1070, asp = 0)

#mapa de las parcelas de la selva baja inundable
my_pal <- c("yellow4", "orange")
tm1 <- tm_shape(spdf_baj) +
  tm_dots(col = "Num_esp", palette = my_pal,
          title = "Número de especies", size = 0.3) +
  tm_text("Num_esp", just = "left", xmod = 0.5, size = 0.7) +
  tm_grid() +
  tm_layout(legend.outside = TRUE, legend.position = c("right", "bottom"))
tm1

#imprimir mapa
tmap_save(tm1, filename = paste(dir.MAPS, "map_parc_bajos.tiff", sep = "/"),
          width = 1720, height = 1070, asp = 0)


##### C.-  definir un grid en donde se realizarán las interpolaciones        ####
#_______________________________________________________________________________

#Visualizar coordenadas máximas y mínimas
extent(rveg)

#ajustar limites para tener la parte central considerando un tamaño de pixel de 30 m
minx1 <- 368985 + 15 
maxx1 <- 377025 - 15
miny1 <- 2089965 + 15
maxy1 <- 2098005 - 15

x1 <- seq(minx1, maxx1, length.out = 268)
y1 <- seq(miny1, maxy1, length.out = 268)
grd <- expand.grid(x = x1, y = y1)


coordinates(grd) = ~x+y

gridded(grd) <- TRUE  # Crear un objeto SpatialPixel 
fullgrid(grd) <- TRUE  # Crear un objeto SpatialGrid 

# se asignan coordenadas a ese grid
proj4string(grd) <- "+proj=utm +zone=16 +units=m +ellps=WGS84 +datum=WGS84 +no_defs"
summary(grd)

#funcion para Calcular RMSE
RMSE <- function(observed, predicted) {
  sqrt(mean((predicted - observed)^2, na.rm = TRUE))
}

#_______________________________________________________________________________
# 2.- Analisis de tendencias de superficies                                 ####
#_______________________________________________________________________________


#### A.- Polinomios de primer orden. Modelo riqueza = bo + b1 x + b2 y      ####     
#_______________________________________________________________________________

# Definir la ecuacion con el polinomio de primer orden

f.1 <- as.formula(Num_esp ~ x + y)

# ejecutar el modelo de regresion
lm.1 <- lm(f.1, data = datos)
summary(lm.1)
dat.1st <- SpatialGridDataFrame(grd, 
                                data.frame(var1.pred = predict(lm.1,
                                                               newdata = grd)))
rgrd   <- raster(dat.1st)
grd.mask <- mask(rgrd, rc, maskvalue = 0) #plot(grd.mask)

#Grafica del mapa interpolado y sitios de muestreo 
my_col <- my_col <- c("yellow4", "orange", "greenyellow",
                      "green","forestgreen","darkgreen")
tm1 <- tm_shape(grd.mask) + 
  tm_raster(style = "cont", n = 7, palette = my_col,
            title = "Número de especies") + 
  tm_shape(spdf) + tm_dots(size = 0.05) +
  tm_grid() +
  tm_layout(legend.outside = TRUE, legend.position = c("right", "bottom"))
tm1

#imprimir mapa
tmap_save(tm1, filename = paste(dir.MAPS,"mapa_pol_1er.tiff", sep = "/"),
          width = 1720, height = 1070, asp = 0)

# guardar en disco el mpa interpolado en formato tiff
writeRaster(grd.mask,
            paste(folder, "Pol_1er.tiff", sep = "/"),
            overwrite = TRUE
)

#### B.- Polinomios de segundo orden                                        ####                                
#_______________________________________________________________________________

# Definir la ecuacion con el polinomio de segundo orden
f.2 <- as.formula(Num_esp ~ x + y + I(x^2) + I(y^2) )

# Ejecutar el modelo de regresion
lm.2 <- lm(f.2, data = datos)
summary(lm.2)
dat.2nd <- SpatialGridDataFrame(grd, 
                                data.frame(var1.pred = predict(lm.2,
                                                               newdata = grd)))
rgrd2   <- raster(dat.2nd)
grd.mask2 <- mask(rgrd2, rc, maskvalue = 0)

#Grafica del mapa interpolado y sitios de muestreo 
my_col <- c("yellow4", "orange","green","darkgreen")
tm1 <- tm_shape(grd.mask2) + 
  tm_raster(style = "cont", n = 10, palette = my_col,
            title = "Número de especies") + 
  tm_shape(spdf) + tm_dots(size = 0.05) +
  tm_grid() +
  tm_layout(legend.outside = TRUE, legend.position = c("right", "bottom"))
tm1

#imprimir mapa
tmap_save(tm1, filename = paste(dir.MAPS,"mapa_pol_2nd.tiff", sep = "/"), 
          width = 1720, height = 1070, asp = 0)

# guardar en disco el mapa creado
writeRaster(grd.mask2,
            paste(folder, "Pol_2nd.tiff", sep = "/"),
            overwrite = TRUE
)

#_______________________________________________________________________________
# 3.-  Prediccion global usando modelos de clasificación                    ####
#_______________________________________________________________________________


#estadisticas descriptivas por tipo de vegetación de vegetación
tapply(datos$Num_esp,datos$t_veg,mean)
tapply(datos$Num_esp,datos$t_veg,sd)

#Diagrama de cajas
#boxplot(Num_esp ~ t_veg, data=datos, col="gray", main="Número de especies", xlab="T Vegetación")

vb <- ggplot(datos, aes(x = as.factor(t_veg), y = Num_esp, 
                        fill = as.factor(t_veg))) + 
  geom_boxplot(show.legend = FALSE) +
  #scale_y_discrete() +
  scale_x_discrete(labels = c("Selva Madura","Selva inter 2", 
                              "Selva inter 1", "Selva joven",
                              "Bosque bajo", "Sabana")) +
  labs(title = "Diagrama de cajas del número de especies",
       x = "Tipos de vegetación", 
       y = "Número de especies") +
  scale_fill_manual(values = c("darkgreen", "forestgreen", "green",
                         "greenyellow", "orange", "yellow4")) +
  theme_bw()
vb

#grabar grafica
ggsave(vb, file = paste(dir.MAPS,"gra_box_nesp.tiff", sep = "/"), 
       width = 5.5, height = 4)

#prueba de normalidad
tapply(datos$Num_esp, datos$t_veg, shapiro.test)

# ANALISIS DE VARIANZA
(model <- aov(Num_esp ~ as.factor(t_veg), data = datos))
#anova(model)
summary(model)

# PRUEBAS POST HOC: Tukey  
prueba <- TukeyHSD(model)
prueba

#bondad de la clasificación U = 1-(varianza dentro de grupos/varianza total)
U <-  1 - (2237.3 / 21482.5)
U

#Mapa de diversidad para cada tipo de vegetación
# todos los valores > 0 and <= 0.9999 son igual 0, etc.
m1 <- c(0, 1, 0 , 1, 2, 34.8, 2,3, 30.9, 3, 4,
        28.7, 4, 5, 15.7, 5, 6, 5.6, 6, 7, 3.1)
mbiomat <- matrix(m1, ncol = 3, byrow = TRUE)
mbio <- reclassify(rveg, mbiomat, right = FALSE)

#Grafica del mapa interpolado y sitios de muestreo 
my_col <- c("white", "yellow4", "orange", "greenyellow",
            "green","forestgreen","darkgreen")
tm1 <- tm_shape(mbio) + 
  tm_raster(style = "cont", n = 6, palette = my_col,
            title = "Número de especies") + 
  tm_shape(spdf) + tm_dots(size = 0.05) +
  tm_grid() +
  tm_layout(legend.outside = TRUE,legend.position = c("right", "bottom"))
tm1

#imprimir mapa
tmap_save(tm1, filename = paste(dir.MAPS,"mapa_clas_glob.tiff", sep = "/"),
          width = 1720, height = 1070, asp = 0)

# guardar en disco el mapa creado
writeRaster(mbio,
            paste(folder, "Clas_glob.tiff", sep = "/"),
            overwrite = TRUE
)

#_______________________________________________________________________________
# 4.- Poligonos de Thiessen                                                 ####
#_______________________________________________________________________________

library(spatstat)  # Used para la funcion dirichlet 
library(maptools)
library(rgeos)

# Crear una superficie teselada, la funcion requiere la libreria maptools
Poli  <-  as(dirichlet(as.ppp(spdf)), "SpatialPolygons") #plot(Poli)

# La funcion dirichlet no contiene informacion sobre la proyeccion 
# Se agrega manualmente 
proj4string(Poli) <- "+proj=utm +zone=16 +units=m +ellps=WGS84 +datum=WGS84 +no_defs"

# Una superficie telesada no almacena los atributos de una capa de puntos
# se utiliza la funcion over() (del paquete sp) para juntar los atributos de puntos
# y de la supeficie telesada por medio de una join
# La funcion () crea un objeto que luego necesita ser agragado al objeto "Poli"
# Con esto se crea un objeto SpatialPolygonsDataFrame 
Poli.z <- over(Poli, spdf, fn = mean)
Poli.spdf  <-  SpatialPolygonsDataFrame(Poli, Poli.z)

#convertir el grid a raster
r <- raster(grd)
rPoli   <- rasterize(Poli.spdf, r, 'Num_esp')  # rasterizar los poligonos
Poli.mask <- mask(rPoli, rc, maskvalue = 0)

#Grafica del mapa interpolado y sitios de muestreo 
my_col <- c("yellow4", "orange", "greenyellow",
            "green","forestgreen","darkgreen")
tm1 <- tm_shape(Poli.mask) + 
  tm_raster(style = "cont", n = 10, palette = my_col, 
            title = "Número de especies") + 
  tm_shape(spdf) + 
  tm_dots(size  = 0.05)  +
  tm_grid() +
  tm_layout(legend.outside = TRUE,legend.position = c("right", "bottom"))
tm1

#imprimir mapa
tmap_save(tm1, filename = paste(dir.MAPS, "mapa_Thissen.tiff", sep = "/"), 
          width = 1720, height = 1070, asp = 0)

# guardar en disco el mapa creado
writeRaster(Poli.mask,
            paste(folder, "thiessen.tiff", sep = "/"),
            overwrite = TRUE
)

#_______________________________________________________________________________
# 5.-  Distancia inversa ponderada (IDW)                                    ####
#_______________________________________________________________________________

library(gstat) # El paquete gstat tiene las rutinas para calcular idw

#Para hacer la funcion de distancia inversa se necesita un objeto SpatialPointDataFrame
#usaremos "spdf"

# Interpolar el grid usando un valor de potencia de 2 (idp=2.0)
Map.idw <- gstat::idw(Num_esp ~ 1, spdf, newdata = grd, idp = 2.0)
rgrd.iwd   <- raster(Map.idw)
grd.mask3 <- mask(rgrd.iwd, rc, maskvalue = 0)

#Grafica del mapa interpolado y sitios de muestreo 
my_col <- c("yellow4", "orange", "greenyellow", 
            "green","forestgreen","darkgreen")
tm1 <- tm_shape(grd.mask3) + 
  tm_raster(style = "cont", n = 10,palette = my_col,
            title = "Número de especies") + 
  tm_shape(spdf) + 
  tm_dots(size = 0.05)  +
  tm_grid() +
  tm_layout(legend.outside = TRUE, legend.position = c("right", "bottom"))
tm1

#imprimir mapa
tmap_save(tm1, filename = paste(dir.MAPS,"mapa_dist_inv2.tiff", sep = "/"), 
          width = 1720, height = 1070, asp = 0)

# guardar en disco el mapa creado
writeRaster(grd.mask3,
            paste(folder, "Dis_inv2.tiff", sep = "/"),
            overwrite = TRUE
)


# Interpolar el grid usando un valor de potencia de 3 (idp=3.0)
Map1.idw <- gstat::idw(Num_esp ~ 1, spdf, newdata = grd, idp = 3.0)
rgrd1.iwd   <- raster(Map1.idw)
grd1.mask3 <- mask(rgrd1.iwd, rc, maskvalue = 0)

#Grafica del mapa interpolado y sitios de muestreo 
my_col <- c("yellow4", "orange", "greenyellow",
            "green","forestgreen","darkgreen")
tm1 <- tm_shape(grd1.mask3) + 
  tm_raster(style = "cont", n = 10,palette = my_col,
            title = "Número de especies") + 
  tm_shape(spdf) + 
  tm_dots(size = 0.05)  +
  tm_grid() +
  tm_layout(legend.outside = TRUE,
            legend.position = c("right", "bottom"))
tm1

#imprimir mapa
tmap_save(tm1, filename = paste(dir.MAPS, "mapa_dist_inv3.tiff", sep = "/"), 
          width = 1720, height = 1070, asp = 0)

# guardar en disco el mapa creado
writeRaster(grd1.mask3,
            paste(folder, "Dis_inv3.tiff", sep = "/"),
            overwrite = TRUE
)

# Validacion cruzada para las funciones de distancia inversa p = 2
IDW.out <- vector(length = length(spdf))
for (i in 1:length(spdf)) {
  IDW.out[i] <- idw(Num_esp ~ 1, spdf[-i,], spdf[i,], idp = 2.0)$var1.pred
}
datos$predd2 <- IDW.out

#calculo del RMSE p 0 2
idwrmse <-  RMSE(datos$Num_esp, datos$predd2)
RMSE_i <- round(idwrmse, digits = 2)

#Modelo de regresión entre prechidos vs observados p 0 2
mod <- lm(datos$predd2 ~ datos$Num_esp)
summary(mod)
r2 <- format(summary(mod)$r.squared, digits = 2)

# Graficar de los valer predichos vs valores observados IDW2
p1 <- ggplot(datos, aes(x = Num_esp, y = predd2)) + 
   geom_point(size = 2, shape = 21 ) +
  geom_rug() +
   geom_smooth(method = lm, se = FALSE, size = 1,color = "red") +
   labs(title = "Distancia inversa p=2",
            x = "Observados", y = "Predichos") +
   expand_limits(x = 0, y = 0) +
   theme_bw() +
   geom_abline(size = 0.7) +
   annotate("text",x = 44, y = 10, 
            label = paste("R2 =", r2), size = 4) +
   annotate("text",x = 45, y = 8, 
            label = paste("RMSE =", RMSE_i), size = 4)
p1 

#grabar grafica
ggsave(p1, file = paste(dir.MAPS, "gra_dis_Idw2.tiff", sep = "/"),
       width = 6, height = 4)

# Validacion cruzada para las funciones de distancia inversa p = 3
IDW1.out <- vector(length = length(spdf))
for (i in 1:length(spdf)) {
  IDW1.out[i] <- idw(Num_esp ~ 1, spdf[-i,], 
                     spdf[i,], idp = 3.0)$var1.pred
}
datos$predd3 <- IDW1.out

#calculo del RMSE p = 3
idwrmse1 <-  RMSE(datos$Num_esp, datos$predd3)
RMSE_i <- round(idwrmse1, digits = 2)

#Modelo de regresión entre prechidos vs observados p = 3
mod1 <- lm(datos$predd3 ~ datos$Num_esp)
summary(mod1)
r2 <- format(summary(mod1)$r.squared, digits = 2)

# Graficar de los valer predichos vs valores observados IDW3
p1 <- ggplot(datos, aes(x = Num_esp, y = predd3)) + 
  geom_point(size = 2, shape = 21) +
  geom_rug() +
  geom_smooth(method = lm, se = FALSE, size = 1,color = "red") +
  labs(title = "Distancia inversa p=3",
       x = "Observados", y = "Predichos") +
  theme_bw() +
  expand_limits(x = 0, y = 0) +
  annotate("text", x = 44, y = 10, label = paste("R2 =", r2), size = 4) +
  annotate("text", x = 45, y = 8, label = paste("RMSE =", RMSE_i), size = 4) +
  geom_abline(size = 0.7)
p1

#grabar grafica
ggsave(p1, file = paste(dir.MAPS, "gra_dis_Idw3.tiff", sep = "/"), 
       width = 6, height = 4)

#_______________________________________________________________________________
# 6.- Interpolacion espacial con Kriging                                    ####
#_______________________________________________________________________________

library(gstat) # El paquete gstat tiene las rutinas para calcular variogramas y kriging

#Estadisticas
datos_selva <- subset(datos, t_veg <= 4)
datos_bajo <- subset(datos, t_veg >= 5)

med <- mean(datos$Num_esp) #Media
m_n <- round(med, digits = 1)
sd <- sd(datos$Num_esp)
s_n <- round(sd, digits = 1)

#Revisar si la variable a interpolar tine una distribución normal
# si no existe normalidad intentar una trasformación log() o sqrt()
p1 <- ggplot(datos, aes(x = Num_esp)) +
  geom_histogram(fill = "blue", color = "black", binwidth = 2) +
  geom_rug() +
  geom_vline(aes(xintercept = mean(Num_esp)), color = "red", lwd = 1.2) +
  geom_vline(aes(xintercept = mean(Num_esp) + sd(Num_esp)),
             color = "forestgreen",  lwd = 1) +
  geom_vline(aes(xintercept = mean(Num_esp) - sd(Num_esp)),
             color = "forestgreen",  lwd = 1) +
  annotate("text", x = 44, y = 19,
           label = paste("Media =", m_n,"D est=", s_n), size = 3) +
  labs(title = "Histograma del número de especies",
       x = "Número de especies", y = "Frecuencia") +
  theme_bw()
p1 
#grabar grafica
ggsave(p1, file = paste(dir.MAPS,"gra_hist1-6.tiff", sep = "/"), 
       width = 6, height = 4)

# prueba de normalidad
shapiro.test(datos$Num_esp)

#crear el variograma experimental
ns.vgm = variogram(Num_esp~1, spdf)
plot(ns.vgm)
#imprimir la grafica
tiff(paste(dir.MAPS, "var_krig1.tiff", sep = "/"), 
     width = 7, height = 4, units = 'in', res = 300)
plot(ns.vgm)
dev.off()

ns.vgm = variogram(Num_esp~1, spdf, width = 350)
plot(ns.vgm)
#imprimir la grafica
tiff(paste(dir.MAPS,"var_krig2.tiff", sep = "/"),
     width = 7, height = 4, units = 'in', res = 300)
plot(ns.vgm)
dev.off()

# Shp esferico, Exp exponencial, Gau Gauciano
# La funcion vgm() tiene como parametros sill, modelo, rango y nugget
ns.fit = fit.variogram(ns.vgm, model = vgm(40, "Sph", 1500, 1))
ns.fit
plot(ns.vgm, ns.fit)
#imprimir la grafica
tiff(paste(dir.MAPS,"var_fitkrig1.tiff", sep = "/"), 
     width = 7, height = 4, units = 'in', res = 300)
plot(ns.vgm, ns.fit)
dev.off()

## Calculo del coeficiente de determinacion del modelo ajustado
## SSErr es la suma ponderara de los cuadrados de los reciduales
SSErr <- attr(ns.fit,"SSErr")
#SStot suma total de cuadrados ponderados
weig <- ns.vgm$np/ns.vgm$dist^2  
#Se utiliza objeto generado en la funcion fit de gstat. 
#El metodo por default usa $N_h/h^2$ con $N_h$ el número de pares de puntos
# y $h$ es la distancia.
SStot <- sum(weig * (ns.vgm$gamma - mean(ns.vgm$gamma))^2)
(R2 <- 1 - SSErr / SStot) #Coeficiente de determinacion

# interpolacion con kriging
ns.k = gstat::krige(Num_esp~1, spdf, grd, model = ns.fit)
rgrd.k   <- raster(ns.k)
grd.mask4 <- mask(rgrd.k, rc, maskvalue = 0)

#Grafica del mapa interpolado y sitios de muestreo 
my_col <- c("yellow4", "orange", "greenyellow",
            "green","forestgreen","darkgreen")
tm1 <- tm_shape(grd.mask4) + 
  tm_raster(style = "cont", n = 10, palette = my_col,
            title = "Número de especies") + 
  tm_shape(spdf) + 
  tm_dots(size = 0.05)  +
  tm_grid() +
  tm_layout(legend.outside = TRUE,legend.position = c("right", "bottom"))
tm1

#imprimir mapa
tmap_save(tm1, filename = paste(dir.MAPS, "mapa_kriging.tiff", sep = "/"),
          width = 1720, height = 1070, asp = 0)

# guardar en disco el mapa creado
writeRaster(grd.mask4,
            paste(folder, "Kriging.tiff", sep = "/"),
            overwrite = TRUE
)

# Desviacion estandar de las interpolaciones con kriging

# obtener elmapa de desviación estandar
rgrd_var.k   <- raster(ns.k, layer = "var1.var")
rgrd_sd.k <- sqrt(rgrd_var.k)
grd.mask5 <- mask(rgrd_sd.k, rc, maskvalue = 0)

#Grafica del mapa interpolado y sitios de muestreo 
my_col <- c("yellow4", "orange", "greenyellow", 
            "green","forestgreen","darkgreen")
tm1 <- tm_shape(grd.mask5) + 
  tm_raster(style = "cont", n = 10, palette = my_col,
            title = "Número de especies") + 
  tm_shape(spdf) + 
  tm_dots(size = 0.05)  +
  tm_grid() +
  tm_layout(legend.outside = TRUE,legend.position = c("right", "bottom"))
tm1

#imprimir mapa
tmap_save(tm1, filename = paste(dir.MAPS,"mapa_sd_kriging.tiff", sep = "/"),
          width = 1720, height = 1070, asp = 0)

# guardar en disco el mapa creado
writeRaster(grd.mask5,
            paste(folder, "SD_Kriging.tiff", sep = "/"),
            overwrite = TRUE
)

# Validacion cruzada para Kriging 
KRG.out <- vector(length = length(spdf))
for (i in 1:length(spdf)) {
   KRG.out[i] <- krige(Num_esp ~ 1, spdf[-i,], spdf[i,], 
                       model = ns.fit)$var1.pred
}
datos$predk <- KRG.out

#calculo del RMSE
krgrmse <-  RMSE(datos$Num_esp, datos$predk)
RMSE_i <- round(krgrmse, digits = 4)

#Modelo de regresión entre prechidos vs observados 
mod <- lm(datos$predk ~ datos$Num_esp)
summary(mod)
r2 <- format(summary(mod)$r.squared, digits = 2)

# Graficar de los valer predichos vs valores observados IDW2
p1 <- ggplot(datos, aes(x = Num_esp, y = predk)) + 
  geom_point(size = 2, shape = 21 ) +
  geom_rug() +
  geom_smooth(method = lm, se = FALSE, size = 1, color = "red") +
  labs(title = "Kriging",
       x = "Observados", y = "Predichos") +
  expand_limits(x = 0, y = 0) +
  theme_bw() +
  geom_abline(size = 0.7) +
  annotate("text",x = 44, y = 12, label = paste("R2 =", r2), size = 4) +
  annotate("text",x = 45, y = 10, label = paste("RMSE =", RMSE_i), size = 4)
p1 

#grabar grafica
ggsave(p1, file = paste(dir.MAPS,"gra_kriging.tiff", sep = "/"),
       width = 6, height = 4)

#_______________________________________________________________________________
# 7.- Interpolacion dentro de estratos                                      ####
#_______________________________________________________________________________

##### A.- Distancia inversa (IDW) dentro de estratos                         ####
#_______________________________________________________________________________


library(gstat) 

# Interpolar el grid usando un valor de potencia de 2 (idp=2.0) para cada estrato
idw.sel <- gstat::idw(Num_esp ~ 1, spdf_sel, newdata = grd, idp = 2.0)
idw.baj <- gstat::idw(Num_esp ~ 1, spdf_baj, newdata = grd, idp = 2.0)

rgrd.iwd_sel   <- raster(idw.sel)
grd.masks <- mask(rgrd.iwd_sel, rs, maskvalue = 0, updatevalue = 0)
rgrd.iwd_baj   <- raster(idw.baj)
grd.maskb <- mask(rgrd.iwd_baj, rb, maskvalue = 0, updatevalue = 0)


grd.masksb <- grd.masks + grd.maskb
grd.mask_todos <- mask(grd.masksb, rc, maskvalue = 0)

#Grafica del mapa interpolado y sitios de muestreo 
my_col <- c("yellow4", "orange", "greenyellow", 
            "green","forestgreen","darkgreen")
tm1 <- tm_shape(grd.mask_todos) + 
  tm_raster(style = "cont", n = 10,palette = my_col, 
            title = "Número de especies") + 
  tm_shape(spdf) + 
  tm_dots(size = 0.05)  +
  tm_grid() +
  tm_layout(legend.outside = TRUE,legend.position = c("right", "bottom"))
tm1

#imprimir mapa
tmap_save(tm1, filename = paste(dir.MAPS,"mapa_iwd_est.tiff", sep = "/"),
          width = 1720, height = 1070, asp = 0)

# guardar en disco el mapa creado
writeRaster(grd.mask_todos,
            paste(folder, "Dis_inv_est.tiff", sep = "/"),
            overwrite = TRUE
)

# Validacion cruzada para las funciones de distancia inversa dentro de estratos
IDW_out.sel <- vector(length = length(spdf_sel))
for (i in 1:length(spdf_sel)) {
  IDW_out.sel[i] <- idw(Num_esp ~ 1, spdf_sel[-i,], 
                        spdf_sel[i,], idp = 2.0)$var1.pred
}
IDW_out.baj <- vector(length = length(spdf_baj))
for (i in 1:length(spdf_baj)) {
  IDW_out.baj[i] <- idw(Num_esp ~ 1, spdf_baj[-i,], 
                        spdf_baj[i,], idp = 2.0)$var1.pred
}

IDW.out_est <- c(IDW_out.sel, IDW_out.baj)

a <- spdf_sel$Num_esp
b <- spdf_baj$Num_esp

obs <- c(a, b)
datos2 <- data.frame(obs, IDW.out_est)

#calculo del RMSE
idw_est.rmse <-  RMSE(datos2$obs, datos2$IDW.out_est)
RMSE_i <- round(idw_est.rmse, digits = 2)

#Modelo de regresión entre prechidos vs observados 
mod1 <- lm(datos2$IDW.out_est ~ datos2$obs)
summary(mod1)
r2 <- format(summary(mod1)$r.squared, digits = 2)

# Graficar de los valer predichos vs valores observados IDW2
p1 <- ggplot(datos2, aes(x = obs, y = IDW.out_est)) + 
  geom_point(size = 2, shape = 21 ) +
  geom_rug() +
  geom_smooth(method = lm, se = FALSE, size = 1, color = "red") +
  labs(title = "Distrancia inversa dentro de estratos",
       x = "Observados", y = "Predichos") +
  expand_limits(x = 0, y = 0) +
    theme_bw() +
  geom_abline(size = 0.7) +
  annotate("text", x = 44, y = 12, label = paste("R2 =", r2), size = 4) +
  annotate("text", x = 45, y = 10, label = paste("RMSE =", RMSE_i), size = 4)
p1 

#grabar grafica
ggsave(p1, file = paste(dir.MAPS,"gra_idw_est.tiff", sep = "/"), 
       width = 6, height = 4)

#_______________________________________________________________________________
##### B.- Interpolacion con kriging dentro de estratos                          ####
#_______________________________________________________________________________

library(gstat)

#Estadisticas
datos_selva <- subset(datos, t_veg <= 4)
datos_bajo <- subset(datos, t_veg >= 5)

med_baj <- mean(datos_bajo$Num_esp) #Media
m_n_baj <- round(med_baj, digits = 1)
sd_baj <- sd(datos_bajo$Num_esp)
s_n_baj <- round(sd_baj, digits = 1)

p1 <- ggplot(datos_bajo, aes(x = Num_esp)) +
  geom_histogram(fill = "blue", color = "black", binwidth = 1) +
  geom_rug() +
  geom_vline(aes(xintercept = mean(Num_esp)), color = "red", lwd = 1.2) +
  geom_vline(aes(xintercept = mean(Num_esp) + sd(Num_esp)),
             color = "forestgreen",  lwd = 1) +
  geom_vline(aes(xintercept = mean(Num_esp) - sd(Num_esp)), 
             color = "forestgreen",  lwd = 1) +
  annotate("text",x = 10, y = 19, 
           label = paste("Media =", m_n_baj,"D est=", s_n_baj), size = 3) +
  labs(title = "Histograma del número de especies en la selva baja inundable",
       x = "Número de especies", y = "Frecuencia") +
  theme_bw()
p1 
#grabar grafica
ggsave(p1, file = paste(dir.MAPS, "gra_hist5-6.tiff", sep = "/"), 
       width = 6, height = 4)

#Estadisticas
med_sel <- mean(datos_selva$Num_esp) #Media
m_n_sel <- round(med_sel, digits = 1)
sd_sel <- sd(datos_selva$Num_esp)
s_n_sel <- round(sd_sel, digits = 1)

#Revisar si la variable a interpolar tine una distribución normal
# si no existe normalidad intentar una trasformación log() o sqrt()
p1 <- ggplot(datos_selva, aes(x = Num_esp)) +
  geom_histogram(fill = "blue", color = "black", binwidth = 3) +
  geom_rug() +
  geom_vline(aes(xintercept = mean(Num_esp)), color = "red", lwd = 1.2) +
  geom_vline(aes(xintercept = mean(Num_esp) + sd(Num_esp)), 
             color = "forestgreen",  lwd = 1) +
  geom_vline(aes(xintercept = mean(Num_esp) - sd(Num_esp)), 
             color = "forestgreen",  lwd = 1) +
  annotate("text",x = 44, y = 19, 
           label = paste("Media =",m_n_sel,"D est=", s_n_sel), size = 3) +
  labs(title = "histograma del número de especies en la selva",
       x = "Número de especies", y = "Frecuencia") +
  theme_bw()
p1 
#grabar grafica
ggsave(p1, file = paste(dir.MAPS,"gra_hist1-4.tiff", sep = "/"), 
       width = 6, height = 4)

#crear los variogramas experimentales para la selva
ns_sel.vgm = variogram(log(Num_esp)~1, spdf_sel)
plot(ns_sel.vgm)
#imprimir la grafica
tiff(paste(dir.MAPS,"var_krig_selva1.tiff", sep = "/"),
     width = 7, height = 4, units = 'in', res = 300)
plot(ns_sel.vgm)
dev.off()

ns_sel.vgm = variogram(log(Num_esp)~1, spdf_sel, width = 600)
ns_sel.vgm
plot(ns_sel.vgm)
#imprimir la grafica
tiff(paste(dir.MAPS, "var_krig_selva2.tiff", sep = "/"), 
     width = 7, height = 4, units = 'in', res = 300)
plot(ns_sel.vgm)
dev.off()

# La funcion vgm() tiene como parametros sill, modelo, rango y nugget
ns_sel.fit = fit.variogram(ns_sel.vgm, model = vgm(0.04, "Exp", 2000, 1))
ns_sel.fit
plot(ns_sel.vgm, ns_sel.fit)
#imprimir la grafica
tiff(paste(dir.MAPS,"var_fitkrig_selva1.tiff", sep = "/"),
     width = 7, height = 4, units = 'in', res = 300)
plot(ns_sel.vgm, ns_sel.fit)
dev.off()

#crear los variogramas experimentales para la selva baja inundable
ns_baj.vgm = variogram(Num_esp~1, spdf_baj)
plot(ns_baj.vgm)
#imprimir la grafica
tiff(paste(dir.MAPS, "var_krig_bajo1.tiff", sep = "/"), 
     width = 7, height = 4, units = 'in', res = 300)
plot(ns_baj.vgm)
dev.off()

ns_baj.vgm = variogram(Num_esp~1, spdf_baj, width = 450)
ns_baj.vgm
plot(ns_baj.vgm)
#imprimir la grafica
tiff(paste(dir.MAPS,"var_krig_bajo2.tiff", sep = "/"), 
     width = 7, height = 4, units = 'in', res = 300)
plot(ns_baj.vgm)
dev.off()

# ajustar semiavariograma en selva baja inundable
ns_baj.fit = fit.variogram(ns_baj.vgm, model = vgm(1, "Sph", 2000, 1))
ns_baj.fit
plot(ns_baj.vgm, ns_baj.fit)
#imprimir la grafica
tiff(paste(dir.MAPS,"var_fitkrig_bajo1.tiff", sep = "/"),
     width = 7, height = 4, units = 'in', res = 300)
plot(ns_baj.vgm, ns_baj.fit)
dev.off()

# interpolacion con kriging para cada estrato
ns_sel.k = gstat::krige(Num_esp~1, spdf_sel, grd, model = ns_sel.fit)
ns_baj.k = gstat::krige(Num_esp~1, spdf_baj, grd, model = ns_baj.fit)

rgrd_baj.k   <- raster(ns_baj.k)
grd.kg_baj <- mask(rgrd_baj.k, rb, maskvalue = 0, updatevalue = 0)
rgrd_sel.k   <- raster(ns_sel.k)
grd.kg_sel <- mask(rgrd_sel.k, rs, maskvalue = 0, updatevalue = 0)


grd.kgsb <- grd.kg_baj + grd.kg_sel
grd.kg_todos <- mask(grd.kgsb, rc, maskvalue = 0)

#Grafica del mapa interpolado y sitios de muestreo 
my_col <- c("yellow4", "orange", "greenyellow",
            "green","forestgreen","darkgreen")
tm1 <- tm_shape(grd.kg_todos) + 
  tm_raster(style = "cont", n = 10,palette = my_col, 
            title = "Número de especies") + 
  tm_shape(spdf) + 
  tm_dots(size = 0.05)  +
  tm_grid() +
  tm_layout(legend.outside = TRUE,legend.position = c("right", "bottom"))
tm1

#imprimir mapa
tmap_save(tm1, filename = paste(dir.MAPS,"mapa_kg_est.tiff", sep = "/"), 
          width = 1720, height = 1070, asp = 0)

# guardar en disco el mapa creado
writeRaster(grd.kg_todos,
            paste(folder, "Int_kri_est.tiff", sep = "/"),
            overwrite = TRUE
)

# Validacion cruzada para las interpoacion con kriging dentro de estratos
KG_out.sel <- vector(length = length(spdf_sel))
for (i in 1:length(spdf_sel)) {
  KG_out.sel[i] <- krige(Num_esp ~ 1, spdf_sel[-i,], spdf_sel[i,], 
                         model = ns_sel.fit)$var1.pred
}
KG_out.baj <- vector(length = length(spdf_baj))
for (i in 1:length(spdf_baj)) {
  KG_out.baj[i] <- krige(Num_esp ~ 1, spdf_baj[-i,], spdf_baj[i,],
                         model = ns_baj.fit)$var1.pred
}

KG.out_est <- c(KG_out.sel, KG_out.baj)
a1 <- datos_selva$Num_esp
b1 <- datos_bajo$Num_esp
KG.obs <- c(a1, b1)
datos2 <- data.frame(KG.obs, KG.out_est)

#calculo del RMSE
kg_est.rmse <-  RMSE(datos2$KG.obs, datos2$KG.out_est)
RMSE_i <- round(kg_est.rmse, digits = 2)

#Modelo de regresión entre prechidos vs observados 
mod2 <- lm(datos2$KG.out_est ~ datos2$KG.obs)
summary(mod2)
r2 <- format(summary(mod2)$r.squared, digits = 2)

# Graficar de los valer predichos vs valores observados IDW2
p1 <- ggplot(datos2, aes(x = KG.obs, y = KG.out_est)) + 
  geom_point(size = 2, shape = 21 ) +
  geom_rug() +
  geom_smooth(method = lm, se = FALSE, size = 1, color = "red") +
  labs(title = "Kriging inversa dentro de estratos",
       x = "Observados", y = "Predichos") +
  expand_limits(x = 0, y = 0) +
  theme_bw() +
  geom_abline(size = 0.7) +
  annotate("text",x = 44, y = 12, label = paste("R2 =", r2), size = 4) +
  annotate("text",x = 45, y = 10, label = paste("RMSE =", RMSE_i), size = 4)
p1 

#grabar grafica
ggsave(p1, file = paste(dir.MAPS, "gra_kg_est.tiff", sep = "/"), 
       width = 6, height = 4)

#_______________________________________________________________________________
# 9 .- Regresion con Kriging                                              ####
#_______________________________________________________________________________

library(gstat)

# Definir la ecuacion del modelo de regresion
f.r <- as.formula(Num_esp ~ TM3+TM4+NDVI+TM3_med+TM3_var+TM3_hom+
                    TM3_con+TM3_dis+TM3_entr+TM3_smo+TM3_cor+TM4_med+
	TM4_var+TM4_hom+TM4_con+TM4_dis+TM4_entr+TM4_smo+TM4_cor+NDVI_med+
	  NDVI_var+NDVI_hom+NDVI_con+NDVI_dis+NDVI_entr+
	NDVI_smo+NDVI_cor)

library(MASS) # Para poder usar la funcion stepAIC

full.model <- lm(f.r, data = datos)
modback <- stepAIC(full.model,direction = "backward")
summary(modback)

f.rb <- as.formula(Num_esp ~ TM3+TM4+NDVI+TM3_med+TM3_con+TM3_dis+TM3_cor+
                    TM4_var+TM4_hom)
back.model <- lm(f.rb, data = datos)
summary(back.model)

#library(olsrr)
# res.1 <- ols_step_best_subset(back.model)
#res.1

#extraer los residuales
f.best <- as.formula(Num_esp ~ TM3+TM4+NDVI+TM3_med+TM3_con+
                       TM3_dis+TM4_var+TM4_hom)
best.model <- lm(f.best, data = datos)
summary(best.model)
spdf$res <- residuals(best.model)


#Creacion del mapa usando el modelo de regresión
# Cargar directorio de imagenes tif
dir.tif <- "D:/met_int/img/"
# Lista de archivos raster
l.tif <- list.files(path = dir.tif,
                    pattern = ".tif$$", 
                    full.names = TRUE)
l.tif

# stack de los archivos raster
r.tif <- stack(l.tif)
r.tif

#calculo del mapa
map.reg <- 52.64670 + (-3.23449 * r.tif$TM3) + (2.62829 * r.tif$TM4) +
  (-374.44328 * r.tif$NDVI) + (-3.19251 * r.tif$TM3_med) +
  (-0.56885 * r.tif$TM3_con) + (10.17215 * r.tif$TM3_dis) +
  (-0.23914 * r.tif$TM4_var) + (22.54901 * r.tif$TM4_hom)


map.regf <- mask(map.reg, rc, maskvalue = 0)

# establecer valores menores a cero como NA
fun <- function(x) { x[x < 0] <- NA; return(x) }
map.regf2 <- calc(map.regf, fun)

#Grafica del mapa interpolado y sitios de muestreo 
my_col <- c("yellow4", "orange", "greenyellow", 
            "green","forestgreen","darkgreen")
tm1 <- tm_shape(map.regf2) + 
  tm_raster(style = "cont", n = 10,palette = my_col, 
            title = "Número de especies") + 
  tm_shape(spdf) + 
  tm_dots(size = 0.05)  +
  tm_grid() +
  tm_layout(legend.outside = TRUE,legend.position = c("right", "bottom"))
tm1

#imprimir mapa
tmap_save(tm1, filename = paste(dir.MAPS,"mapa_regresion.tiff", sep = "/"),
          width = 1720, height = 1070, asp = 0)

# guardar en disco el mapa creado
writeRaster(map.regf2,
            paste(folder, "regresion.tiff", sep = "/"),
            overwrite = TRUE
)

# validacion cruzada para regresion
n <- 0
reg.out <- as.vector(0)
for (i in 1:nrow(datos)) {
  n <- 1 + n
  model <- lm(Num_esp ~ TM3+TM4+NDVI+TM3_med+TM3_con+TM3_dis+
                TM4_var+TM4_hom,data = datos[-n,])
  reg.out[i] <- model$coefficients[1] + (model$coefficients[2] * datos$TM3[i]) +
    (model$coefficients[3]*datos$TM4[i]) +
    (model$coefficients[4]*datos$NDVI[i]) +
    (model$coefficients[5]*datos$TM3_med[i]) +
    (model$coefficients[6]*datos$TM3_con[i]) +
    (model$coefficients[7]*datos$TM3_dis[i]) +
    (model$coefficients[8]*datos$TM4_var[i]) +
    (model$coefficients[9]*datos$TM4_hom[i])
}
datos$predreg <- reg.out

#calculo del RMSE 
reg.rmse <-  RMSE(datos$Num_esp, datos$predreg)
RMSE_i <- round(reg.rmse, digits = 2)

#Modelo de regresión entre prechidos vs observados para regresion multiple
mod <- lm(datos$predreg ~ datos$Num_esp)
summary(mod)
r2 <- format(summary(mod)$r.squared, digits = 2)

# Graficar de los valer predichos vs valores observados IDW2
p1 <- ggplot(datos, aes(x = Num_esp, y = predreg)) + 
  geom_point(size = 2, shape = 21 ) +
  geom_rug() +
  geom_smooth(method = lm, se = FALSE, size = 1,color = "red") +
  expand_limits(x = 0, y = 0) +
  labs(title = "Regresión",
       x = "Observados", y = "Predichos") +
  theme_bw() + 
  geom_abline(size = 0.7) +
  annotate("text",x = 44, y = 10, label = paste("R2 =", r2), size = 4) +
  annotate("text",x = 45, y = 8, label = paste("RMSE =", RMSE_i), size = 4)
p1 

#grabar grafica
ggsave(p1, file = paste(dir.MAPS, "gra_regresion.tiff", sep = "/"), 
       width = 6, height = 4)

#crear el variograma experimental
res.vgm = variogram(res~1, spdf)
plot(res.vgm)
#imprimir la grafica
tiff(paste(dir.MAPS,"var_krig_reg1.tiff", sep = "/"),
     width = 7, height = 4, units = 'in', res = 300)
plot(res.vgm)
dev.off()

res.vgm = variogram(res~1, spdf, width = 350)
plot(res.vgm)
#imprimir la grafica
tiff(paste(dir.MAPS,"var_krig_reg2.tiff", sep = "/"),
     width = 7, height = 4, units = 'in', res = 300)
plot(res.vgm)
dev.off()

# Ajustar una funcion al variograma experimental
res.fit = fit.variogram(res.vgm, model = vgm(30, "Exp", 1500, 1))
res.fit
plot(res.vgm, res.fit)
#imprimir la grafica
tiff(paste(dir.MAPS,"var_fitkrig_reg1.tiff", sep = "/"), 
     width = 7, height = 4, units = 'in', res = 300)
plot(res.vgm, res.fit)
dev.off()

# interpolacion de los residalues
res.k = gstat::krige(res~1, spdf, grd, model = res.fit)
res_grd.k   <- raster(res.k)
grd.res <- mask(res_grd.k, rc, maskvalue = 0)

#Grafica del mapa residuales 
my_col <- c("yellow4", "orange", "greenyellow", 
            "green","forestgreen","darkgreen")
tm1 <- tm_shape(grd.res) + 
  tm_raster(style = "cont", n = 10,palette = my_col,
            midpoint = NA,
            title = "Residuales en regresión") + 
  tm_grid() +
  tm_layout(legend.outside = TRUE,legend.position = c("right", "bottom"))
tm1

#imprimir mapa
tmap_save(tm1, filename = paste(dir.MAPS,"mapa_res_regresion.tiff", sep = "/"), 
          width = 1720, height = 1070, asp = 0)


#Función para remplazar valores nulos con 0 en raster
replaceNA <- function(x, na.rm, ...){
  if (is.na(x[1]))
    return(0)
  else
    return(x)
}

raster.reg <- calc(map.regf2, fun = replaceNA)
raster.res <- calc(grd.res, fun = replaceNA)

# mapa de regresion y suma de residuales regresion con kriging
reg_kri <- raster.reg + raster.res
map.reg_kri <- mask(reg_kri, rc, maskvalue = 0)

map.reg_kri2 <- reclassify(map.reg_kri, 
                           cbind(-Inf, 0, NA), 
                           right = FALSE) #sustituir negativos con NA



#Grafica del mapa interpolado y sitios de muestreo 
my_col <- c("yellow4", "orange", "greenyellow", 
            "green","forestgreen","darkgreen")
tm1 <- tm_shape(map.reg_kri2) + 
  tm_raster(style = "cont", n = 10, palette = my_col,
            title = "Regresión con Kriging") + 
  tm_shape(spdf) + 
  tm_dots(size = 0.05)  +
  tm_grid() +
  tm_layout(legend.outside = TRUE,legend.position = c("right", "bottom"))
tm1

#imprimir mapa
tmap_save(tm1, filename = paste(dir.MAPS,"mapa_regresion_kri.tiff", sep = "/"),
          width = 1720, height = 1070, asp = 0)

# guardar en disco el mapa creado
writeRaster(map.reg_kri2,
            paste(folder, "Reg_con_kri.tiff", sep = "/"),
            overwrite = TRUE
)

# Validacion cruzada de residuales 
res.out <- vector(length = length(spdf))
for (i in 1:length(spdf)) {
  res.out[i] <- krige(res ~ 1, spdf[-i,], spdf[i,], model = res.fit)$var1.pred
}
#vector de predichos  mas residuales: regresion con kriging
datos$predrk <- reg.out + res.out

#calculo del RMSE para  regresion con kriging
rekg.rmse <-  RMSE(datos$Num_esp, datos$predrk)
RMSE_i <- round(rekg.rmse, digits = 2)

#Modelo de regresión entre prechidos vs observados para regresion con kriging
mod1 <- lm(datos$predrk ~ datos$Num_esp)
summary(mod1)
r2 <- format(summary(mod1)$r.squared, digits = 2)

# Graficar de los valer predichos vs valores observados IDW2
p1 <- ggplot(datos, aes(x = Num_esp, y = predrk)) + 
  geom_point(size = 2, shape = 21 ) +
  geom_rug() +
  geom_smooth(method = lm, se = FALSE, size = 1, color = "red") +  
  expand_limits(x = 0, y = 0) +
  labs(title = "Regresión con Kriging",
       x = "Observados", y = "Predichos") +
  theme_bw() +
  geom_abline(size = 0.7) +
  annotate("text",x = 44, y = 10, label = paste("R2 =", r2), size = 4) +
  annotate("text",x = 45, y = 8, label = paste("RMSE =", RMSE_i), size = 4)
p1 

#grabar grafica
ggsave(p1, file = paste(dir.MAPS,"gra_regre_Krig.tiff", sep = "/"),
       width = 6, height = 4)

#_______________________________________________________________________________
# 10 .- Random forest de regresion con Kriging                             ####
#_______________________________________________________________________________

library(gstat)
#Cargar paquetes que se utilizaran en este modulo
library(randomForest) 
library(ModelMap) 
library(raster)
library(sp)
#library(spatial.tools)

# Definición de tipos de archivos 
qdatafn <- "dat.csv" #archivo de datos

#cuando la valicacion se hace con un grupo de datos independientes
#trainfn <- "d_train.csv" # archivo de datos para entrenar el modelo
#testfn <- "d_test.csv" # archivo de datos para validar el modelo
# Los datos e entrenamiento y validacion seran iguales 
#get.test(proportion.test=0.3,
#         qdatafn=qdatafn,
#         seed=49,
#         folder=folder,
#         qdata.trainfn=trainfn,
#         qdata.testfn=testfn)

# Definición de los parametros del modelo
mtype <- "RF" # el Modelo es de Random Forest
MODELfn <- "Nesp"
predList <- c( "TM3", "TM4", "NDVI", "TM3_med", "TM3_var", "TM3_hom", "TM3_con",   
               "TM3_dis", "TM3_entr", "TM3_smo", "TM3_cor", "TM4_med", "TM4_var",
               "TM4_hom", "TM4_con", "TM4_dis", "TM4_entr", "TM4_smo", "TM4_cor",
               "NDVI_med", "NDVI_var", "NDVI_hom", "NDVI_con", "NDVI_dis",
               "NDVI_entr", "NDVI_smo", "NDVI_cor")
predFactor <- FALSE
resname <-  "Num_esp" # variable de respuesta
restype <- "continuous"
seed <- 25
urowname <- "ID"  #identificador unico de cada renglon 

# Construcción del modelo usando Random Forest
model.sp <- model.build( model.type = mtype,
                              qdata.trainfn = qdatafn,
                              folder = folder,
                              unique.rowname = urowname,
                              MODELfn = MODELfn,
                              predList = predList,
                              predFactor = predFactor,
                              response.name = resname,
                              response.type = restype,
                              seed = seed,
                              na.action = "na.omit")

model.sp

# Debido a que  qdata.testfn=FALSE se realiza una validación cruzada
# que se indica en prediction.type="CV"
val.sp <- model.diagnostics( model.obj = model.sp,
                                     qdata.trainfn = qdatafn,
                                     qdata.testfn = FALSE,
                                     folder = folder,
                                     MODELfn = MODELfn,
                                     unique.rowname = urowname,
                                     #se realiza una validación cruzada
                                     prediction.type = "CV",
                                     device.type = c("pdf"),
                                     cex = 1.2)
head(val.sp)

# obtencion del mapa

#Directorio donde se encuentran las imagenes predictores
dir.img <- "imgrf"
dir.img <- paste(folder,dir.img,sep = "/")

# Crear una lista con las imagenes predictoras
l.img <- list.files(path = dir.img, pattern = ".img$")
l.img

# crear un data frame de imagenes 
rastLUTfn <- data.frame()

for (i in 1:length(l.img)) {
  rastLUTfn[i,1] <- paste(dir.img, l.img[i], sep = "/")
  rastLUTfn[i,2] <- sub(".img$", "", l.img[i])
  rastLUTfn[i,3] <- 1 #número de bandas
}


rastLUTfn

# Definir el nombre del mapa de salida
OUTPUTfn <- "map_RF.img"

# Crear una imagen con el mapa del Num esp predicho
model.mapmake( model.obj = model.sp,
               folder = folder,		
               rastLUTfn = rastLUTfn,
               # paramétros del mapa						
               OUTPUTfn = OUTPUTfn,
               MODELfn = MODELfn,
               map.sd = TRUE
)

map.rf <- raster(OUTPUTfn)
map.rarf <- mask(map.rf, rc, maskvalue = 0)

#Grafica del mapa interpolado y sitios de muestreo 
my_col <- c("yellow4", "orange", "greenyellow",
            "green","forestgreen","darkgreen")
tm1 <- tm_shape(map.rarf) + 
  tm_raster(style = "cont", n = 10, palette = my_col,
            title = "Número de especies") + 
  tm_shape(spdf) + 
  tm_dots(size = 0.05)  +
  tm_grid() +
  tm_layout(legend.outside = TRUE,legend.position = c("right", "bottom"))
tm1

#imprimir mapa
tmap_save(tm1, filename = paste(dir.MAPS, "mapa_randomforest.tiff", sep = "/"), 
          width = 1720, height = 1070, asp = 0)

# guardar en disco el mapa creado
writeRaster(map.rarf,
            paste(folder, "randomf.tiff", sep = "/"),
            overwrite = TRUE
)

# Calculo de Residuales del modelo random forest
spdf$resrf <- datos$Num_esp - model.sp$predicted 

#crear el variograma experimental
res.rf_vgm = variogram(resrf~1, spdf)
plot(res.rf_vgm)
#imprimir la grafica
tiff(paste(dir.MAPS, "var_krig_rf1.tiff", sep = "/"), 
     width = 7, height = 4, units = 'in', res = 300)
plot(res.rf_vgm)
dev.off()

res.rf_vgm = variogram(resrf~1, spdf, width = 350)
plot(res.rf_vgm)
#imprimir la grafica
tiff(paste(dir.MAPS, "var_krig_rf2.tiff", sep = "/"), 
     width = 7, height = 4, units = 'in', res = 300)
plot(res.rf_vgm)
dev.off()

# Ajustar una funcion al variograma experimental
res.rf_fit = fit.variogram(res.rf_vgm, model = vgm(50, "Exp", 1200, 1))
res.rf_fit
plot(res.rf_vgm, res.rf_fit)
#imprimir la grafica
tiff(paste(dir.MAPS, "var_fitkrig_rf1.tiff", sep = "/"), 
     width = 7, height = 4, units = 'in', res = 300)
plot(res.rf_vgm, res.rf_fit)
dev.off()

# interpolacion de los residalues
res.rf = gstat::krige(resrf~1, spdf, grd, model = res.rf_fit)
#spplot(res.rf["var1.pred"],main = "Reciduales RF - Kriging " )

res_grd.rf   <- raster(res.rf)
grd.res2 <- mask(res_grd.rf, rc, maskvalue = 0)

#Grafica del mapa de residuales 
my_col <- c("yellow4", "orange", "greenyellow", 
            "green","forestgreen","darkgreen")
tm1 <- tm_shape(grd.res2) + 
  tm_raster(style = "cont", n = 10,palette = my_col,
            midpoint = NA,
            title = "Residuales en Random Forest") + 
  tm_grid() +
  tm_layout(legend.outside = TRUE, legend.position = c("right", "bottom"))
tm1

#imprimir mapa
tmap_save(tm1, filename = paste(dir.MAPS, "mapa_res_randomf.tiff", sep = "/"), 
          width = 1720, height = 1070, asp = 0)

# Validacion cruzada de residuales 
res1.out <- vector(length = length(spdf))
for (i in 1:length(spdf)) {
  res1.out[i] <- krige(resrf ~ 1, spdf[-i,], spdf[i,], 
                       model = res.rf_fit)$var1.pred
}

# mapa de RF y suma de residuales, random forest con kriging
rf_kri <- map.rarf + grd.res2

#Grafica del mapa interpolado y sitios de muestreo 
my_col <- c("yellow4", "orange", "greenyellow", 
            "green","forestgreen","darkgreen")
tm1 <- tm_shape(rf_kri) + 
  tm_raster(style = "cont", n = 10,palette = my_col, 
            title = "Random Forest con Kriging") + 
  tm_shape(spdf) + tm_dots(size = 0.05)  +
  tm_grid() +
  tm_layout(legend.outside = TRUE,legend.position = c("right", "bottom"))
tm1

#imprimir mapa
tmap_save(tm1, filename = paste(dir.MAPS,"mapa_randomf_kri.tiff", sep = "/"), 
          width = 1720, height = 1070, asp = 0)

# guardar en disco el mapa creado
writeRaster(rf_kri,
            paste(folder, "RF_con_kri.tiff", sep = "/"),
            overwrite = TRUE
)

#vector de predichos mas residuales: rf con kriging
val.sp$predrf <- val.sp$pred + res1.out

#calculo del RMSE para el modelo de random forest 
rf.rmse <-  RMSE(val.sp$obs, val.sp$pred)
RMSE_i <- round(rf.rmse, digits = 2)
mod <- lm(val.sp$pred ~ val.sp$obs)
summary(mod)
r2 <- format(summary(mod)$r.squared, digits = 2)

# Graficar de los valer predichos vs valores observados random forest
p1 <- ggplot(val.sp, aes(x = obs, y = pred)) + 
  geom_point(size = 2, shape = 21 ) +
  geom_rug() +
  geom_smooth(method = lm, se = FALSE, size = 1, color = "red") +
  labs(title = "Random Forest",
       x = "Observados", y = "Predichos") +
  expand_limits(x = 0, y = 0) +
  theme_bw() +
  geom_abline(size = 0.7) +
  annotate("text",x = 44, y = 10, label = paste("R2 =", r2), size = 4) +
  annotate("text",x = 45, y = 8, label = paste("RMSE =", RMSE_i), size = 4)
p1 

#grabar grafica
ggsave(p1, file = paste(dir.MAPS, "gra_randomforest.tiff", sep = "/"),
       width = 6, height = 4)

#calculo del RMSE para el modelo de random forest con kriging 
rf_kg.rmse <-  RMSE(val.sp$obs, val.sp$predrf)
RMSE_i <- round(rf_kg.rmse, digits = 2)
mod1 <- lm(val.sp$predrf ~ spdf$Num_esp)
summary(mod1)

r2 <- format(summary(mod1)$r.squared, digits = 2)

# Graficar de los valer predichos vs valores observados Randon forest con kriging
p1 <- ggplot(val.sp, aes(x = obs, y = predrf)) + 
  geom_point(size = 2, shape = 21 ) +
  geom_rug() +
  geom_smooth(method = lm, se = FALSE, size = 1,color = "red") +
  labs(title = "Random Forest con Kriging",
       x = "Observados", y = "Predichos") +
  expand_limits(x = 0, y = 0) +
  theme_bw() +
  geom_abline(size = 0.7) +
  annotate("text",x = 44, y = 10, label = paste("R2 =", r2), size = 4) +
  annotate("text",x = 45, y = 8, label = paste("RMSE =", RMSE_i), size = 4)
p1 

#grabar grafica
ggsave(p1, file = paste(dir.MAPS,"gra_randomf_Krig.tiff", sep = "/"),
       width = 6, height = 4)

 
